<?php
/**
 * (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

use Motive\Woocommerce\Config;
use Motive\Woocommerce\LanguageManager;
use Motive\Woocommerce\Token;
use Motive\Woocommerce\TimeLimit;
use Motive\Woocommerce\Builder\FeedBuilder;
use Motive\Woocommerce\Builder\InfoBuilder;
use Motive\Woocommerce\Builder\SchemaBuilder;

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.motive.co
 * @since      1.0.0
 *
 * @package    Motive
 * @subpackage Motive/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Motive
 * @subpackage Motive/public
 * @author     Motive <motive@motive.co>
 */
class Motive_Public {
	const NS                         = 'motive';
	const HEADER_MOTIVE_TOKEN        = 'X_MOTIVE_TOKEN';
	const MOTIVEX_URL                = 'https://assets.motive.co/motive-x/';
	const MOTIVE_DATA_POST_TYPE      = 'motive_data_post';
	const MOTIVE_DATA_SHOPPER_PRICES = 'motive_data_shopper_prices';

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;
	}

	/**
	 * Register a new post_type and creates a post of the type to create shopperPrices endpoint.
	 */
	public function register_motive_post_type() {
		if ( ! post_type_exists( self::MOTIVE_DATA_POST_TYPE ) ) {
			register_post_type(
				self::MOTIVE_DATA_POST_TYPE,
				array(
					'public'             => false,
					'publicly_queryable' => true,
				)
			);
			flush_rewrite_rules();
		}
		if ( ! self::get_motive_post_by_title( self::MOTIVE_DATA_SHOPPER_PRICES ) ) {
			wp_insert_post(
				array(
					'post_type'   => self::MOTIVE_DATA_POST_TYPE,
					'post_status' => 'publish',
					'post_title'  => self::MOTIVE_DATA_SHOPPER_PRICES,
				)
			);
		}
	}

	private function get_motive_post_by_title( $title ) {
		return get_page_by_title( $title, OBJECT, self::MOTIVE_DATA_POST_TYPE );
	}

	/**
	 * When wp select the template to show, this function intercepts it to give our data template
	 * if post_type is the one we're using.
	 */
	public function motive_post_type_template( $single_template ) {
		global $post;
		if ( self::MOTIVE_DATA_POST_TYPE === $post->post_type ) {
			$single_template = __DIR__ . '/templates/motive-data-template.php';
		}
		return $single_template;
	}

	/**
	 * This function adds to add-to-cart return the number of elements in cart.
	 */
	public function add_cart_count_fragment( $fragments ) {
		if ( Config::is_configured() ) {
			$fragments['meta#motive_cart_info'] = sprintf( '<meta id="motive_cart_info" data-products-count="%s" />', WC()->cart->cart_contents_count );
		}
		return $fragments;
	}

	/**
	 * Add preload links in head to improve configuration and layer load. These links are also used
	 * to obtain both urls front-side. Only will append if MCS is configured.
	 */
	public function preload_links() {
		if ( ! Config::is_configured() ) {
			return;
		}
		printf( "<link id='motive-layer-js' rel='preload' href='%s' as='script' crossorigin='anonymous'>", esc_url( self::MOTIVEX_URL . 'v2/app.js' ) );

		$rel_config_url = 'preload';
		if ( Config::get_layer_init_params_inline() ) {
			$rel_config_url = 'motive-config';
		}
		printf( "<link id='motive-config-url' rel='%s' href='%s' as='fetch'>", esc_attr( $rel_config_url ), esc_url( get_rest_url( null, 'motive/front-config' ) ) );
	}

	/**
	 * Include layer initialization if MCS is configured.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {
		if ( ! Config::is_configured() ) {
			return;
		}

		if ( Config::get_layer_init_params_inline() ) {
			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/motive.js', array( 'jquery' ), $this->version, false );
			wp_add_inline_script( $this->plugin_name, $this->get_js_as_string() . ';', 'after' );
		} else {
			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/motive-public.js', array( 'jquery' ), $this->version, false );
		}
		wp_add_inline_script( $this->plugin_name, 'const motive = ' . wp_json_encode( $this->get_layer_init_params() ) . ';', 'before' );
		wp_add_inline_script( $this->plugin_name, 'const motiveShopperPricesNonce = ' . wp_json_encode( wp_create_nonce( 'motive-endpoint' ) ) . ';', 'before' );

	}

	/**
	 * Register public endpoints.
	 *
	 * @since    0.0.1
	 */
	public function add_endpoints() {
		// route url: domain.com/wp-json/motive/check
		// route url: domain.com/?rest_route=/motive/check
		register_rest_route(
			self::NS,
			'check',
			array(
				'methods'             => 'GET',
				'permission_callback' => '__return_true',
				'callback'            => function () {
					return array(
						'status' => 'ok',
					);
				},
			)
		);

		// route url: domain.com/wp-json/motive/info
		// route url: domain.com/?rest_route=/motive/info
		register_rest_route(
			self::NS,
			'info',
			array(
				'methods'             => 'GET',
				'permission_callback' => array( 'Motive_Public', 'motive_permission_callback' ),
				'callback'            => function () {
					return InfoBuilder::build();
				},
			)
		);

		// route url: domain.com/wp-json/motive/<lang>/schema
		// route url: domain.com/?rest_route=/motive/<lang>/schema
		register_rest_route(
			self::NS,
			'(?P<lang>.+)/schema',
			array(
				'methods'             => 'GET',
				'permission_callback' => array( 'Motive_Public', 'motive_permission_callback' ),
				'callback'            => function ( $data ) {
					$lang         = LanguageManager::get_instance( $data->get_param( 'lang' ) );
					$schema_builder = new SchemaBuilder( $lang, get_woocommerce_currency() );
					return $schema_builder->build( $data );
				},
			)
		);

		// route url: domain.com/wp-json/motive/config
		// route url: domain.com/?rest_route=/motive/config
		register_rest_route(
			self::NS,
			'config',
			array(
				'methods'             => 'GET',
				'permission_callback' => array( 'Motive_Public', 'motive_permission_callback' ),
				'callback'            => function () {
					return Config::export();
				},
			)
		);
		// route url: domain.com/wp-json/motive/config
		// route url: domain.com/?rest_route=/motive/config
		register_rest_route(
			self::NS,
			'config',
			array(
				'methods'             => 'POST',
				'permission_callback' => array( 'Motive_Public', 'motive_permission_callback' ),
				'callback'            => function ( $data ) {
					$body = $data->get_json_params();
					if ( ! is_array( $body ) ) {
						$response = new WP_REST_Response( array( 'error' => 'Bad request body.' ) );
						$response->set_status( 400 );
						return $response;
					}
					return Config::import( $body );
				},
			)
		);

		// If MCS is not configured, there is no need to add following endpoint, related to layer.
		if ( ! Config::is_configured() ) {
			return;
		}

		// route url: domain.com/wp-json/motive/front-config
		// route url: domain.com/?rest_route=/motive/front-config
		register_rest_route(
			self::NS,
			'front-config',
			array(
				'methods'             => 'GET',
				'permission_callback' => '__return_true',
				'callback'            => function () {
					if ( ! Config::is_configured() ) {
						return;
					}
					return $this->get_layer_init_params();
				},
			)
		);
	}

	/**
	 * Filters the pre-calculated result of a REST API dispatch request.
	 * Here we are using it to check if error reporting should be enabled, taking in care our configuration.
	 * We will only modify response in case a configuration is enabled, activate_error_reporting
	 *
	 * @param mixed           $result  Response to replace the requested version with. Can be anything
	 *                                 a normal endpoint can return, or null to not hijack the request.
	 * @param WP_REST_Server  $server  Server instance.
	 * @param WP_REST_Request $request Request used to generate the response.
	 */
	public function add_debug_interceptor( $result, $server, $request ) {
		// If `activate_error_reporting` configuration value is not "1", we don't do anything.
		if ( Config::get_activate_error_reporting() !== '1' ) {
			return null;
		}
		// If request don't relate to a motive route, we stop here and let wp to fully manage request.
		if ( strpos( $request->get_route(), '/motive/' ) === false ) {
			return null;
		}
		$query = $request->get_params();
		// phpcs:disable
		// Only if we are in a motive related route, we will check if a "debug" query param is present.
		if ( isset( $query['debug'] ) ) {
			// If debug query param is present, we will enable error reporting to check what can be 
			// happening with our endpoints in json response.
			ini_set( 'display_errors', '1' );
			error_reporting( E_ALL );
		} else {
			// In case debug query param is not present, as we want to avoid warnings, notices or infos 
			// which can break json, we will disable "display_errors"
			ini_set( 'display_errors', '0' );
		}
		// phpcs:enable
		return null;
	}


	/**
	 * This function intercepts WordPress rest engine before headers or any info is sent, allowing
	 * us to send data using echo.
	 *
	 * @param bool $served - Boolean given by WordPress which will be used to know if the request was served.
	 * @param WP_HTTP_Response $result - Standard response which can be used to send info (other endpoints use it internally).
	 * @param WP_REST_Request $request - Object with current request info.
	 * @param WP_REST_Server $server - Server instance
	 *
	 * @return boolean indicating if any content was served.
	 */
	public function intercept_for_feed_endpoint( $served, $result, $request, $server ) {
		$route = $request->get_route();
		// route url: domain.com/wp-json/motive/<lang>/feed
		// route url: domain.com/?rest_route=/motive/<lang>/feed
		if ( ! preg_match( '/\/motive\/([^\/]+)\/feed/', $route, $matches ) ) {
			return false;
		}

		if ( ! self::motive_permission_callback( $request ) ) {
			header( 'HTTP/1.1 401 Unauthorized' );
			return true;
		}
		header( 'HTTP/1.1 200 Ok' );

		$lang         = LanguageManager::get_instance( $matches[1] );
		$feed_builder = new FeedBuilder( $lang, get_woocommerce_currency() );

		$query_params = $request->get_params();
		$from_id      = 0;
		if ( ! empty( $query_params['from_id'] ) ) {
			$from_id = (int) $query_params['from_id'];
		}
		$max_time = Config::get_time_limit();
		if ( ! empty( $query_params['max_time'] ) ) {
			$max_time = (int) $query_params['max_time'];
		}
		$feed       = $feed_builder->build( $from_id );
		$time_limit = new TimeLimit( $max_time, $max_time );
		$last_id    = null;

		$feed->jsonStream(
			0,
			function ( $id ) use ( $time_limit, &$last_id, $lang, $query_params ) {
				$id_product = (int) $id; // Id can have variant separator '-'
				// Split feed between variants not supported
				if ( $last_id === $id_product ) {
						return false;
				}
				$last_id = $id_product;
				if ( $time_limit->remaining_time() > 5 ) {
						return false;
				}
				$query_params['from_id'] = $last_id;

				$rest_url      = get_rest_url( null, "motive/$lang->current_lang/feed" );
				$next_page_url = add_query_arg( $query_params, $rest_url );
				return $next_page_url;
			}
		);
		return true;
	}


	/**
	 * Utility function to manage token comprobation.
	 */
	public static function motive_permission_callback( $request ) {
		return Token::check( $request->get_header( self::HEADER_MOTIVE_TOKEN ) );
	}


	/**
	 * Returns layer initialization data.
	 */
	private function get_layer_init_params() {
		$lang_manager = LanguageManager::get_instance();
		if ( ! function_exists( 'wc_get_cart_url' ) ) {
			require_once ABSPATH . 'wp-content/plugins/woocommerce/includes/wc-core-functions.php';
		}

		$motive = array(
			'initParams' => array(
				'xEngineId'       => Config::get_engine_id()[ $lang_manager->current_lang ],
				'lang'            => $lang_manager->current_lang,
				'currency'        => get_woocommerce_currency(),
				'triggerSelector' => Config::get_trigger_selector(),
				'isolated'        => (bool) Config::get_layer_isolated(),
				'cartUrl'         => wc_get_cart_url(),
			),
			'options'    => array(
				'shopperPrices' => (bool) Config::get_shopper_prices(),
			),
			'endpoints'  => array(
				'shopperPrices' => get_permalink( self::get_motive_post_by_title( self::MOTIVE_DATA_SHOPPER_PRICES ) ),
			),
		);

		return $motive;
	}

	/**
	 * Returns js content as string. Required build step
	 */
	private function get_js_as_string() {
		require_once __DIR__ . '/js/motive-public.js.php';
		return $js;
	}
}
