<?php
/**
 * (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

namespace Motive\Woocommerce\Builder;

use Motive\Woocommerce\Config;
use Motive\Woocommerce\Model\Attribute;
use Motive\Woocommerce\Model\Feature;

if ( ! defined( 'WC_VERSION' ) ) {
	exit;
}

class AttributeBuilder {

	protected $language_manager;

	public $attributes = array();
	public $features   = array();
	private $features_limit;
	private $simple_attribute_query;

	public function __construct( $language_manager ) {
		$this->language_manager       = $language_manager;
		$this->features_limit         = intval( Config::get_features_limit() );
		$this->simple_attribute_query = 1 === intval( Config::get_simple_attribute_query() );
	}

	/**
	 * Fetch all attributes & features into local public vars
	 */
	public function fetch() {
		if ( $this->simple_attribute_query ) {
			return $this->fetch_attributes_from_taxonomies();
		}

		return $this->fetch_attributes_from_products();
	}

	/**
	 * Parse product list to obtain all local & global attributes & features
	 */
	private function fetch_attributes_from_products() {
		global $wpdb;

		$post_meta_sql            = PostMetaSqlBuilder::get_post_meta_sql( array( '_product_attributes' ), $wpdb->get_blog_prefix(), 'INNER' );
		$relevant_taxonomies_join = TaxonomyBuilder::get_relevant_taxonomies_join();

		$inner_join_langs = $this->language_manager->get_inner_join_products_lang();

		$prefix             = $wpdb->get_blog_prefix();
		$raw_products_metas = $wpdb->get_results(
			"
      SELECT $post_meta_sql->select_statement, COALESCE( variable.object_id, 0) AS is_attribute_from_variable_product
      FROM {$prefix}posts p
      $inner_join_langs
      $post_meta_sql->join_statement
      $relevant_taxonomies_join
      WHERE p.post_type='PRODUCT' AND exclude_from_search.object_id IS NULL
    "
		);

		$features        = array();
		$attributes      = array();
		$feature_counter = array();
		foreach ( $raw_products_metas as $raw_product_meta ) {
			$product_attributes = maybe_unserialize( $raw_product_meta->p_product_attributes );
			if ( ! is_array( $product_attributes ) && ! is_object( $product_attributes ) ) {
				continue;
			}
			foreach ( $product_attributes as $id => $product_attribute ) {
				$id   = sanitize_title( $id );
				$name = $this->language_manager->get_label( $id, $product_attribute );
				if ( $product_attribute['is_variation'] && '0' !== $raw_product_meta->is_attribute_from_variable_product ) {
					if ( empty( $attributes[ $id ] ) ) {
						$attributes[ $id ] = new Attribute( self::get_attribute_key( $id ), $name, false );
					}
				} else {
					if ( empty( $features[ $id ] ) ) {
						$features[ $id ]        = new Feature( self::get_feature_key( $id ), $name );
						$feature_counter[ $id ] = 1;
					} else {
						$feature_counter[ $id ]++;
					}
				}
			}
		}
		$this->finalize_attributes_retrieval( $attributes, $features, $feature_counter );
	}

	/**
	 * Parse all taxonomies starting by 'pa_'. Only able to retrieve global taxonomies.
	 */
	private function fetch_attributes_from_taxonomies() {
		global $wpdb;
		$prefix = $wpdb->get_blog_prefix();

		$taxonomies_applying_as_attribute = $wpdb->get_col( "SELECT meta_key FROM {$prefix}postmeta WHERE meta_key LIKE 'attribute_%' GROUP BY meta_key" );

		$product_taxonomies = $wpdb->get_results( "SELECT taxonomy as 'taxonomy', SUM(count) as 'count' FROM {$prefix}term_taxonomy WHERE taxonomy LIKE 'pa_%' GROUP BY taxonomy" );

		$features        = array();
		$attributes      = array();
		$feature_counter = array();
		foreach ( $product_taxonomies as $product_taxonomy ) {
			$id   = sanitize_title( $product_taxonomy->taxonomy );
			$name = $this->language_manager->get_label( $id, array( 'is_taxonomy' => true ) );

			$features[ $id ]        = new Feature( self::get_feature_key( $id ), $name );
			$feature_counter[ $id ] = intval( $product_taxonomy->count );

			if ( in_array( "attribute_$product_taxonomy->taxonomy", $taxonomies_applying_as_attribute, true ) ) {
				$attributes[ $id ] = new Attribute( self::get_attribute_key( $id ), $name, false );
			}
		}

		$this->finalize_attributes_retrieval( $attributes, $features, $feature_counter );
	}

	private function finalize_attributes_retrieval( $attributes, $features, $feature_counter ) {
		if ( $this->features_limit > 0 && count( $feature_counter ) > $this->features_limit ) {
			arsort( $feature_counter, SORT_NUMERIC );
			$top_used_features = array_slice( $feature_counter, 0, $this->features_limit );
			$features          = array_intersect_key( $features, $top_used_features );
		}

		$this->attributes = array_values( $attributes );
		$this->features   = array_values( $features );
	}

	/**
	 * Creates an unique identifier for features
	 *
	 * @param string $id entity id
	 * @return string
	 */
	public static function get_feature_key( $id ) {
		return "f$id";
	}


	/**
	 * Creates an unique identifier for attributes
	 *
	 * @param string $id entity id
	 * @return string
	 */
	public static function get_attribute_key( $id ) {
		return "a$id";
	}
}
