<?php
/**
 * (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 *
 * This file is part of Motive Commerce Search.
 *
 * This file is licensed to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * @author Motive (motive.co)
 * @copyright (C) 2023 Motive Commerce Search Corp S.L. <info@motive.co>
 * @license https://www.apache.org/licenses/LICENSE-2.0 Apache License, Version 2.0
 */

namespace Motive\Woocommerce;

if ( ! defined( 'WC_VERSION' ) ) {
	exit;
}
/**
 * Class Config
 * Enum and util class for all module configs
 * @package Motive\Prestashop
 * @method static set_token(string $randomToken)
 * @method static get_token()
 * @method static set_trigger_selector(string $selector)
 * @method static get_trigger_selector()
 * @method static set_engine_id(array $engineId)
 * @method static get_engine_id($id_lang)
 * @method static set_time_limit(string $value)
 * @method static get_time_limit()
 * @method static set_image_size(string $value)
 * @method static getImageSize()
 * @method static set_features_limit(string $value)
 * @method static get_features_limit()
 * @method static set_activate_error_reporting(string $value)
 * @method static get_activate_error_reporting()
 * @method static set_layer_isolated(string $value)
 * @method static get_layer_isolated()
 * @method static set_shopper_prices(string $boolNum)
 * @method static get_shopper_prices()
 * @method static set_layer_init_params_inline(string $boolNum)
 * @method static get_layer_init_params_inline()
 */
class Config {

	const PREFIX     = 'MOTIVE_';
	const DEFINITION = array(
		'trigger_selector'         => array(),
		'engine_id'                => array(
			'lang' => true,
		),
		'time_limit'               => array(
			'default' => '30',
		),
		'image_size'               => array(
			'default' => 'woocommerce_thumbnail',
		),
		'features_limit'           => array(
			'default' => '100',
		),
		'activate_error_reporting' => array(
			'default' => 'off',
		),
		'layer_isolated'           => array(
			'default' => '1',
		),
		'shopper_prices'           => array(
			'default' => '1',
		),
		'last_sync_info'           => array(
			'hidden' => true,
			'lang'   => true,
		),
		'layer_init_params_inline' => array(
			'default' => '1',
		),
		'image_limit'              => array(
			'default' => '10',
		),
		'simple_attribute_query'   => array(
			'default' => '0',
		),
	);

	// Configs Enum
	const TRIGGER_SELECTOR         = 'MOTIVE_TRIGGER_SELECTOR';
	const ENGINE_ID                = 'MOTIVE_ENGINE_ID';
	const TIME_LIMIT               = 'MOTIVE_TIME_LIMIT';
	const IMAGE_SIZE               = 'MOTIVE_IMAGE_SIZE';
	const FEATURES_LIMIT           = 'MOTIVE_FEATURES_LIMIT';
	const ACTIVATE_ERROR_REPORTING = 'MOTIVE_ACTIVATE_ERROR_REPORTING';
	const LAYER_ISOLATED           = 'MOTIVE_LAYER_ISOLATED';
	const SHOPPER_PRICES           = 'MOTIVE_SHOPPER_PRICES';
	const LAST_SYNC_INFO           = 'MOTIVE_LAST_SYNC_INFO';
	const LAYER_INIT_PARAMS_INLINE = 'MOTIVE_LAYER_INIT_PARAMS_INLINE';
	const IMAGE_LIMIT              = 'MOTIVE_IMAGE_LIMIT';
	const SIMPLE_ATTRIBUTE_QUERY   = 'MOTIVE_SIMPLE_ATTRIBUTE_QUERY';
	/**
	 * Install defined configurations
	 * @return bool
	 */
	public static function install() {
		$lang_manager = LanguageManager::get_instance();
		$result       = true;
		foreach ( static::DEFINITION as $name => $properties ) {
			$key = static::name2key( $name );
			if ( static::is( $name, 'persistent' ) && get_option( $key ) !== false ) {
				continue;
			}
			$default_value = empty( $properties['default'] ) ? '' : $properties['default'];
			$value         = empty( get_option( $key ) ) ? $default_value : get_option( $key );
			if ( static::is( $name, 'lang' ) ) {
				$key   = static::name2key( $name, $lang_manager->default_lang );
				$value = empty( get_option( $key ) ) ? '' : get_option( $key );
			}
			$result = update_option( $key, $value ) && $result;
		}
		return $result;
	}

	/**
	 * Uninstall defined configurations
	 */
	public static function uninstall() {
		$result = true;
		foreach ( array_keys( static::DEFINITION ) as $name ) {
			if ( ! static::is( $name, 'persistent' ) ) {
				$result = delete_option( $key ) && $result;
			}
		}
		return $result;
	}

	/**
	 * Magic method to handle get and set methods over configuration entries.
	 * @param $method
	 * @param $args
	 * @return mixed
	 * @throws Exception
	 */
	public static function __callStatic( $method, $args ) {
		$operation = substr( $method, 0, 3 );
		$name      = substr( $method, 4 );
		if ( array_key_exists( $name, static::DEFINITION ) ) {
			if ( static::is( $name, 'lang' ) ) {
				return static::call_static_lang( $operation, $name, $args );
			}
			$key = static::name2key( $name );
			if ( 'get' === $operation ) {
				return get_option( $key );
			} elseif ( 'set' === $operation ) {
				return update_option( $key, $args[0] );
			}
		}

		return null;
	}

	/**
	 * Manages lang based configuration get/set.
	 * @param $operation
	 * @param $name
	 * @param $args
	 * @return mixed
	 * @throws Exception
	 */
	private static function call_static_lang( $operation, $name, $args = null ) {
		$results = array();
		foreach ( LanguageManager::get_instance()->langs as $lang ) {
			$key = static::name2key( $name, $lang );
			if ( 'get' === $operation ) {
				$results[ $lang ] = get_option( $key );
			} elseif ( 'set' === $operation && null !== $args && array_key_exists( $lang, $args[0] ) ) {
				return update_option( $key, $args[0][ $lang ] );
			}
		}
		return $results;
	}


	/**
	 * Update module configurations with the values in $config
	 * @param array $new_config
	 * @return array
	 */
	public static function import( array $new_config ) {
		$errors = array();
		foreach ( $new_config as $name => $value ) {
			try {
				static::import_one( $name, $value );
			} catch ( ConfigException $e ) {
				$errors[] = $e->getMessage();
			}
		}
		return $errors;
	}

	/**
	 * Import single configuration value
	 * @param $name
	 * @param $value
	 * @return string|null
	 * @throws ConfigException
	 */
	protected static function import_one( $name, $value ) {
		if ( ! array_key_exists( $name, static::DEFINITION ) ) {
			throw new ConfigException( "Invalid configuration name '$name'" );
		}

		if ( ! is_array( $value ) ) {
			update_option( static::name2key( $name ), $value );
			return;
		}

		if ( is_array( $value ) !== static::is( $name, 'lang' ) ) {
			throw new ConfigException( "Invalid configuration value for '$name'" );
		}
		$bad_iso_codes = array();
		foreach ( $value as $iso_code => $val ) {
			if ( ! in_array( $iso_code, LanguageManager::get_instance()->langs, true ) ) {
				$bad_iso_codes[] = $iso_code;
			}
			update_option( static::name2key( $name, $iso_code ), $value[ $iso_code ] );
		}
		if ( ! empty( $bad_iso_codes ) ) {
			$bad_iso_codes = implode( ', ', $bad_iso_codes );
			throw new ConfigException( "Invalid lang(s) '$bad_iso_codes' for config '$name'" );
		}
		return null;
	}

	/**
	 * Export module configuration to array.
	 * @param bool $all include all configs, also hidden ones
	 * @return array
	 */
	public static function export( $all = false ) {
		$result = array();
		foreach ( array_keys( static::DEFINITION ) as $name ) {
			if ( ! $all && static::is( $name, 'hidden' ) ) {
				continue;
			}

			if ( static::is( $name, 'lang' ) ) {
				$result[ $name ] = array();

				foreach ( LanguageManager::get_instance()->langs as $lang ) {
					$value = get_option( static::name2key( $name, $lang ) );
					if ( false !== $value ) {
						$result[ $name ][ $lang ] = $value;
					}
				}

				if ( 0 === count( $result[ $name ] ) ) {
					$result[ $name ] = '';
				}
			} else {
				$result[ $name ] = get_option( static::name2key( $name ) );
			}
		}
		return $result;
	}

	/**
	 * Check if a property of config is true, because empty() does not work on class constants
	 * @param $name
	 * @param $property
	 * @return bool
	 */
	protected static function is( $name, $property ) {
		return array_key_exists( $name, static::DEFINITION )
			&& array_key_exists( $property, static::DEFINITION[ $name ] )
			&& true === static::DEFINITION[ $name ][ $property ];
	}

	/**
	 * Convert config name to Key format
	 * @param $name
	 * @param $lang
	 * @return string
	 */
	protected static function name2key( $name, $lang = null ) {
		return static::PREFIX . strtoupper( $name ) . ( null !== $lang ? '-' . strtoupper( $lang ) : '' );
	}

	/**
	 * Returns true if motive search layer is configured, which is checking non-emptyness engine id
	 * for current lang && trigger selector.
	 *
	 * @return boolean
	 */
	public static function is_configured() {
		return ! empty( self::get_engine_id()[ LanguageManager::get_instance()->default_lang ] )
			&& ! empty( self::get_trigger_selector() );
	}
}
